



/***************************************************************************
 * open_port()
 *	This routine is used to interface from the System V "open" format
 *	to the serial driver for the dos port we have provided.
 *
 *	Because of the interface to the driver, this routine does not
 *	do the final open of the port.	Instead, it sets up a data structure
 *	that is filled with pertinent data.  The data structure is further
 *	filled by subsequent ioctl() calls.  When the program actually
 *	requests a read or write, the the port is implicitly reopened with 
 *	the correct values.
 *
 *	Returns:
 *		I/O register base value; -1 if error
 ***************************************************************************/
int
open_port(port_id, mode)
    char *port_id;		/* Name of serial port to open	*/
    int  mode;		/* mode parameter not used	*/
{

	if (port_id == NULL){
		return -1;
	}

	if ((strlen(port_id) == 4)
	&& (!strcspn(port_id, "com") || !strcspn(port_id, "COM"))) {
		strcpy(serial_device, port_id);
		serial_iobase = 0;
		serial_ivec = 0;
		serial_baud = B9600>>16;
		serial_opened = 1;
		serial_setup = 1;
		serial_updated = 0 ;
		return serial_open() == -1 ? 0 : (int)serial_iobase;
	}
	return open(port_id, mode);
}

static int
serial_check()
{
	if (!serial_setup){
		return 0;
	}

	if (serial_updated) {
		close_port(serial_iobase);
		serial_setup = 1;
	}
	if (!serial_opened) {
		serial_opened = 1;
		if (serial_open() == -1){
			return 0;

		}
	}
	return 1;
}


/***************************************************************************
 * read_port()
 *	This routine is used to read from the serial port of an MS/DOS
 *	system.  It interfaces to a driver that handles all port communication.
 *
 *	First, the routine checks to see if the port has been opened for
 *	use yet.  If not, it opens the device and prepares it for reading.
 *	If so, it passes the information on to the serial driver to actually
 *	get the data.
 *
 *	Returns:
 *		int number of bytes read, or -1 if error
 ***************************************************************************/
int
read_port(devindex, buffer, n, timeout)
    unsigned int devindex;	/* Index into device array for port	*/
    char *buffer;		/* Buffer for data read from port	*/
    int n;		/* Number of bytes to read	*/
    int timeout;		/* Timeout value in seconds	*/
				    /* The low lever driver is			*/
				    /* expecting milliseconds.			*/
{
	if (!buffer){
		return -1;
	} else if (is_serial(devindex)) {
		return serial_check() ? 
			serial_read(serial_iobase, buffer, n, 1000*timeout):-1;
	} else {
		/* Not the serial device, do normal I/O */
		if (devindex == fileno(stdin)){
			return -2;
		} else {
			read(devindex, buffer, n);
		}
	}
}


/***************************************************************************
 * write_port()
 *	This routine is used to write to the serial port of an MS/DOS
 *	system.  It interfaces to a driver that handles all port communication.
 *
 *	First, the routine checks to see if the port has been opened for
 *	use yet.  If not, it opens the device and prepares it for writing.
 *	If so, it passes the information on to the serial driver to actually
 *	send the data.
 *
 *	Returns:
 *		int  number of bytes actually written; -1 if error.
 ***************************************************************************/
int
write_port(devindex, buffer, n)
    unsigned int devindex;	/* Index into device array for port	*/
    char *buffer;		/* Buffer of data to write to port	*/
    int n;		/* Number of bytes to write	*/
{
	if (!buffer){
		return -1;
	} else if (is_serial(devindex)) {
		return serial_check() ?
				serial_write(serial_iobase,buffer,n) : -1;
	} else {
		/* Not the serial device */
		return write(devindex, buffer, n);
	}
}
/***************************************************************************
 * close_port()
 *	This routine is used to interface from the System V "close" format
 *	to the serial driver for the dos port we have provided.
 ***************************************************************************/
close_port(devindex)
    unsigned int devindex;	/* serial port to close */
{
	if (is_serial(devindex)) {
		serial_close();
		serial_opened = 0;
		serial_setup = 0;
		serial_updated = 0;
	} else {
		close(devindex);
	}
}

